
external g_type_init : unit -> unit = "g_type_init_stub"

let _ = 
  g_type_init ()

type size_fun = int -> int -> int * int

let round f =
  int_of_float (if f < 0. then f -. 0.5 else f +. 0.5)

let default w h = (w, h)
let at_size rw rh w h = 
  (if rw < 0 then w else rw), (if rh < 0 then h else rh)
let at_zoom zx zy w h =
  if w < 0 || h < 0
  then (w, h)
  else
    (round (float w *. zx)), (round (float h *. zy))
let at_max_size mw mh w h =
  if w < 0 || h < 0
  then (w, h)
  else
    let zx = float mw /. float w in
    let zy = float mh /. float h in
    let z = min zx zy in
    (round (float w *. z)), (round (float h *. z))

let at_zoom_with_max zx zy mw mh w h =
  if w < 0 || h < 0
  then (w, h)
  else 
    let rw = round (float w *. zx) in
    let rh = round (float h *. zy) in
    if rw > mw || rh > mh
    then 
      let zx = float mw /. float w in
      let zy = float mh /. float h in
      let z = min zx zy in
      (round (float w *. z)), (round (float h *. z))
    else
      (rw, rh)

external set_default_dpi : float -> unit = "rsvg_set_default_dpi_stub"


type t
external new_handle : gz:bool -> (int -> int -> int * int) -> t
    = "rsvg_handle_new_stub"
external free_handle : t -> unit
    = "rsvg_handle_free_stub"
external write : t -> string -> off:int -> len:int -> unit = "rsvg_handle_write_stub"
external close : t -> unit = "rsvg_handle_close_stub"
external get_surface : t -> Sdlvideo.surface = "rsvg_handle_get_pixbuf_stub"
external set_dpi : t -> float -> unit = "rsvg_handle_set_dpi_stub"

let render ?(gz=false) ?dpi ?(size_cb=default) buff =
  let h = new_handle ~gz size_cb in
  begin match dpi with
  | None -> ()
  | Some v -> set_dpi h v end ;
  write h buff 0 (String.length buff) ;
  close h ;
  let s = get_surface h in
  free_handle h ; 
  s

let render_from_file ?gz ?dpi ?size_cb fname =
  let ic = open_in fname in
  let len = in_channel_length ic in
  let buff = String.create len in
  really_input ic buff 0 len ;
  close_in ic ;
  render ?gz ?dpi ?size_cb buff
